//--------------------------------------------------------------------------------------
// File: Trilight.fx
//
// The effect file for the Trilight sample.  
// 
// Copyright (c) Microsoft Corporation. All rights reserved.
//--------------------------------------------------------------------------------------


//--------------------------------------------------------------------------------------
// Global variables
//--------------------------------------------------------------------------------------
float4 g_MaterialAmbientColor;      // Material's ambient color
float4 g_MaterialDiffuseColor;      // Material's diffuse color
int g_nNumLights;

float3 g_LightDir;                  // Light's direction in world space
float4 g_LightColour[3];            // The three colours making up the light.

texture g_MeshTexture;              // Color texture for mesh

float    g_fTime;                   // App's time in seconds
float4x4 g_mWorld;                  // World matrix for object
float4x4 g_mWorldViewProjection;    // World * View * Projection matrix

float g_fLightFactor1;
float g_fLightFactor2;


//--------------------------------------------------------------------------------------
// Texture samplers
//--------------------------------------------------------------------------------------
sampler MeshTextureSampler = 
sampler_state
{
    Texture = <g_MeshTexture>;
    MipFilter = LINEAR;
    MinFilter = LINEAR;
    MagFilter = LINEAR;
};


//--------------------------------------------------------------------------------------
// Vertex shader output structure
//--------------------------------------------------------------------------------------
struct VS_OUTPUT
{
    float4 Position   : POSITION;   // vertex position 
    float4 Diffuse    : COLOR0;     // vertex diffuse color (note that COLOR0 is clamped from 0..1)
    float2 TextureUV  : TEXCOORD0;  // vertex texture coords 
};


//--------------------------------------------------------------------------------------
// This shader computes standard transform and lighting
//--------------------------------------------------------------------------------------
VS_OUTPUT RenderSceneVS( float4 vPos : POSITION, 
                         float3 vNormal : NORMAL,
                         float2 vTexCoord0 : TEXCOORD0,
                         uniform int nNumLights,
                         uniform bool bWrapAroundLighting,
                         uniform bool bHemisphricalLighting )
{
    VS_OUTPUT Output;
    float3 vNormalWorldSpace;
  
    // Transform the position from object space to homogeneous projection space
    Output.Position = mul(vPos, g_mWorldViewProjection);
    
    // Transform the normal from object space to world space    
    vNormalWorldSpace = normalize(mul(vNormal, (float3x3)g_mWorld)); // normal (world space)
    
    // Compute simple directional lighting equation
    float3 vTotalLightDiffuse = float3(0,0,0);
    float LdotN = dot(vNormalWorldSpace, g_LightDir);
    if ( bWrapAroundLighting )
    {
	    vTotalLightDiffuse +=
			  ( g_LightColour[0] * max(0,LdotN + g_fLightFactor1) * g_fLightFactor2 );
    }
    else if ( bHemisphricalLighting )
    {
	    //vTotalLightDiffuse += g_LightColour[2] +
		//	  ( ( g_LightColour[0] - g_LightColour[2] ) * ( LdotN + 1.0 ) * 0.5 );
		vTotalLightDiffuse += ( g_LightColour[0] + g_LightColour[2] ) * 0.5
				+ ( g_LightColour[0] - g_LightColour[2] ) * LdotN * 0.5;
    }
    else
    {
		// Trilighting
	    vTotalLightDiffuse +=
			  ( g_LightColour[0] * max(0,LdotN) )
			+ ( g_LightColour[1] * (1-abs(LdotN)) )
			+ ( g_LightColour[2] * max(0,-LdotN) );
	}
        
    Output.Diffuse.rgb = g_MaterialDiffuseColor * vTotalLightDiffuse;   
    Output.Diffuse.a = 1.0f; 
    
    Output.TextureUV = 0; 
    
    return Output;    
}


//--------------------------------------------------------------------------------------
// Pixel shader output structure
//--------------------------------------------------------------------------------------
struct PS_OUTPUT
{
    float4 RGBColor : COLOR0;  // Pixel color    
};


//--------------------------------------------------------------------------------------
// This shader outputs the pixel's color by modulating the texture's
//       color with diffuse material color
//--------------------------------------------------------------------------------------
PS_OUTPUT RenderScenePS( VS_OUTPUT In,
                         uniform bool bTexture ) 
{ 
    PS_OUTPUT Output;

    // Lookup mesh texture and modulate it with diffuse
    if( bTexture )
        Output.RGBColor = tex2D(MeshTextureSampler, In.TextureUV) * In.Diffuse;
    else
        Output.RGBColor = In.Diffuse;

    return Output;
}


//--------------------------------------------------------------------------------------
// Renders scene to render target
//--------------------------------------------------------------------------------------

technique RenderSceneTrilight
{
    pass P0
    {          
        VertexShader = compile vs_2_0 RenderSceneVS( 1, false, false );
        PixelShader  = compile ps_2_0 RenderScenePS( false );
    }
}

technique RenderSceneWraparound
{
    pass P0
    {          
        VertexShader = compile vs_2_0 RenderSceneVS( 1, true, false );
        PixelShader  = compile ps_2_0 RenderScenePS( false );
    }
}

technique RenderSceneHemispherical
{
    pass P0
    {          
        VertexShader = compile vs_2_0 RenderSceneVS( 1, false, true );
        PixelShader  = compile ps_2_0 RenderScenePS( false );
    }
}


